<?php

/**
 * @file
 * Administration functionality for the PoolParty Taxonomy Manager.
 */


/**
 * List all saved PoolParty Taxonomy Manager configurations.
 *
 * @return string
 *   The rendered HTML of the list of configurations.
 */
function pp_taxonomy_manager_admin_list() {
  $output = array();

  $output['pp_taxonomy_manager_title'] = array(
    '#type' => 'markup',
    '#markup' => '<h3 class="semantic-connector-table-title">' . t('PoolParty Taxonomy Manager configurations') . '</h3>',
  );

  $rows = array();
  $configs = pp_taxonomy_manager_config_load_multiple();
  /* @var $config PPTaxonomyManagerConfig */
  foreach ($configs as $config) {
    // Set the title.
    $title = '<div class="semantic-connector-led" data-server-id="' . $config->getConnection()
        ->getId() . '" data-server-type="pp-server" title="' . t('Checking service') . '"></div>';
    $title .= $config->getTitle();

    // Set the link to the PoolParty server.
    $pp_server = l($config->getConnection()
        ->getTitle(), $config->getConnection()
        ->getUrl() . '/PoolParty', array('attributes' => array('target' => '_blank')));

    // Set the project label.
    $project_id = $config->getProjectID();
    $connection_config = $config->getConnection()->getConfig();
    $project_label = 'Project label not found';
    if (isset($connection_config['projects'])) {
      foreach ($connection_config['projects'] as $project) {
        if ($project->id == $project_id) {
          $project_label = $project->title;
          break;
        }
      }
    }

    // Create the list of the interconnected taxonomies.
    $configuration = $config->getConfig();
    $vocabularies = taxonomy_vocabulary_load_multiple(array_keys($configuration['taxonomies']));
    $taxonomies = array();
    foreach ($vocabularies as $vocabulary) {
      $taxonomies[] = l($vocabulary->name, 'admin/structure/taxonomy/' . $vocabulary->machine_name);
    }
    $taxonomies = empty($taxonomies) ? '<div class="semantic-connector-italic">' . t('not yet set') . '</div>' : theme('item_list', array('items' => $taxonomies));

    // Set the operations.
    $actions = array(
      l(t('Edit'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId()),
      l(t('Delete'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/delete'),
    );

    // Create the row.
    $rows[] = array(
      $title,
      $pp_server,
      $project_label,
      $taxonomies,
      implode(' | ', $actions),
    );
  }

  $output['pp_taxonomy_manager'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('Name'),
      t('PoolParty server'),
      t('Selected project'),
      t('Interconnected taxonomies'),
      t('Operations'),
    ),
    '#rows' => $rows,
    '#attributes' => array(
      'id' => 'pp-taxonomy-manager-configurations-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $output['#attached']['js'] = array(libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js');
  }

  return $output;
}


/**
 * Ajax callback function for checking if a new PoolParty server is available.
 */
function pp_taxonomy_manager_new_available($form, $form_state) {
  $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
  $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';

  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Create a new connection (without saving) with the current form data.
    $connection = SemanticConnector::getConnection('pp_server');
    $connection->setUrl($form_state['values']['url']);
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));

    $availability = $connection->getApi('PPT')->available();
    if (isset($availability['message']) && !empty($availability['message'])) {
      return '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . $availability['message'] . '</div>';
    }
    else {
      return $availability['success'] ? $available : $not_available;
    }
  }

  return $not_available;
}

/**
 * The form for a PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   The PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_connection_form($form, &$form_state, PPTaxonomyManagerConfig $config = NULL) {
  if (!is_null($config) && !empty($config)) {
    $is_configuration_new = FALSE;
    $form['tmid'] = array(
      '#type' => 'hidden',
      '#value' => $config->getId(),
    );

    $form['title'] = array(
      '#type' => 'hidden',
      '#value' => $config->getTitle(),
    );
  }
  else {
    $is_configuration_new = TRUE;
    $config = new PPTaxonomyManagerConfig();

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#description' => t('Name of the PoolParty Taxonomy Manager configuration.'),
      '#size' => 35,
      '#maxlength' => 255,
      '#default_value' => $config->getTitle(),
      '#required' => TRUE,
      '#validated' => TRUE,
    );
  }

  // Container: Server settings.
  $form['server_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('1. Select the PoolParty server to use'),
  );

  $connections = SemanticConnector::getConnectionsByType('pp_server');
  if (!empty($connections)) {
    $connection_options = array();
    /* @var $connection SemanticConnectorConnection */
    foreach ($connections as $connection) {
      $credentials = $connection->getCredentials();
      $connection_options[implode('|', array(
        $connection->getTitle(),
        $connection->getUrl(),
        $credentials['username'],
        $credentials['password'],
      ))] = $connection->getTitle();
    }
    $form['server_settings']['load_connection'] = array(
      '#type' => 'select',
      '#title' => t('Load an available PoolParty server'),
      '#options' => $connection_options,
      '#empty_option' => '',
      '#default_value' => '',
    );
  }

  // Container: Connection details.
  $form['server_settings']['connection_details'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection details'),
  );

  $connection = $config->getConnection();
  $form['server_settings']['connection_details']['connection_id'] = array(
    '#type' => 'hidden',
    '#value' => $connection->getId(),
  );

  $form['server_settings']['connection_details']['server_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Server title'),
    '#description' => t('A short title for the server below.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $connection->getTitle(),
    '#required' => TRUE,
  );

  $form['server_settings']['connection_details']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('URL, where the PoolParty server runs.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $connection->getUrl(),
    '#required' => TRUE,
  );

  $credentials = $connection->getCredentials();
  $form['server_settings']['connection_details']['credentials'] = array(
    '#type' => 'fieldset',
    '#title' => t('Credentials'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['server_settings']['connection_details']['credentials']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Name of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $credentials['username'],
  );
  $form['server_settings']['connection_details']['credentials']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#description' => t('Password of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 128,
    '#default_value' => $credentials['password'],
  );

  $form['server_settings']['health_check'] = array(
    '#type' => 'button',
    '#value' => t('Health check'),
    '#ajax' => array(
      'callback' => 'pp_taxonomy_manager_new_available',
      'wrapper' => 'health_info',
      'method' => 'replace',
      'effect' => 'slide',
    ),
  );

  if ($is_configuration_new) {
    $markup = '<div id="health_info">' . t('Click to check if the server is available.') . '</div>';
  }
  else {
    $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
    $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';
    $markup = $connection->available() ? $available : $not_available;
  }
  $form['server_settings']['health_info'] = array(
    '#markup' => $markup,
  );

  // Container: Project loading.
  $form['project_load'] = array(
    '#type' => 'fieldset',
    '#title' => t('2. Load the projects'),
  );

  $form['project_load']['load_projects'] = array(
    '#type' => 'button',
    '#value' => 'Load projects',
    '#ajax' => array(
      'event' => 'click',
      'callback' => 'pp_taxonomy_manager_connection_form_get_projects',
      'wrapper' => 'projects-replace',
    ),
  );

  // Container: Project selection.
  $form['project_select'] = array(
    '#type' => 'fieldset',
    '#title' => t('3. Select the project to use'),
    '#description' => t('Note: In case this list is still empty after clicking the "Load projects" button make sure that a connection to the PoolParty server can be established and check the rights of your selected user inside PoolParty.'),
  );

  // Get the project options for the currently configured PoolParty server.
  $project_options = array();
  if (!$is_configuration_new) {
    $projects = $connection->getApi('PPT')->getProjects();
    foreach ($projects as $project) {
      $project_options[$project->id] = $project->title;
    }
  }
  // Configuration set admin page.
  $form['project_select']['project'] = array(
    '#type' => 'select',
    '#title' => 'Select a project',
    '#prefix' => '<div id="projects-replace">',
    '#suffix' => '</div>',
    '#options' => $project_options,
    '#default_value' => (!$is_configuration_new ? $config->getProjectId() : NULL),
    '#required' => TRUE,
    '#validated' => TRUE,
  );

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => ($is_configuration_new ? t('Save and edit details') : t('Save')),
    '#prefix' => '<div class="admin-form-submit-buttons" style="margin-top: 40px;">',
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => ($is_configuration_new ? 'admin/config/semantic-drupal/pp-taxonomy-manager' : 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId()),
    '#suffix' => '</div>',
  );

  // Add CSS and JS files
  $form['#attached'] = array(
    'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
  );

  return $form;
}


/**
 * Ajax callback function to get a project select list.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 *
 * @return array
 *   The select form element containing the project options for the current
 *   PoolParty server connection.
 */
function pp_taxonomy_manager_connection_form_get_projects($form, $form_state) {
  $projects_element = $form['project_select']['project'];

  $project_options = array();
  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Create a new connection (without saving) with the current form data.
    $connection = SemanticConnector::getConnection('pp_server');
    $connection->setUrl($form_state['values']['url']);
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));

    $projects = $connection->getApi('PPT')->getProjects();
    foreach ($projects as $project) {
      $project_options[$project->id] = $project->title;
    }
  }

  $projects_element['#options'] = $project_options;
  return $projects_element;
}

/**
 * The validation handler for the PoolParty Taxonomy Manager connection form.
 */
function pp_taxonomy_manager_connection_form_validate($form, &$form_state) {
  // Only do project validation during the save-operation, not during
  // AJAX-requests like the health check of the server.
  if ($form_state['triggering_element']['#parents'][0] == 'save') {
    // A title is required.
    if (!isset($form_state['values']['tmid']) && empty($form_state['values']['title'])) {
      form_set_error('title', t('Name field is required.'));
    }

    // A project needs to be selected.
    if (empty($form_state['values']['project'])) {
      form_set_error('project', t('Please select a project.'));
    }
    // And it has to be a valid project (one that is available on the connected
    // PoolParty server).
    else {
      if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
        // Create a new connection (without saving) with the current form data.
        $connection = SemanticConnector::getConnection('pp_server');
        $connection->setUrl($form_state['values']['url']);
        $connection->setCredentials(array(
          'username' => $form_state['values']['name'],
          'password' => $form_state['values']['pass'],
        ));
        $projects = $connection->getApi('PPT')->getProjects();
        $project_is_valid = FALSE;
        foreach ($projects as $project) {
          if ($project->id == $form_state['values']['project']) {
            $project_is_valid = TRUE;
            break;
          }
        }
        if (!$project_is_valid) {
          form_set_error('project', t('The selected project is not available on the given PoolParty server configuration.'));
        }
      }
      else {
        form_set_error('url', t('The field URL must be a valid URL.'));
      }
    }
  }
}

/**
 * The submit handler for the PowerTagging connection form.
 */
function pp_taxonomy_manager_connection_form_submit($form, &$form_state) {
  // Always create a new connection, if URL and type are the same the old one
  // will be used anyway.
  $connection = SemanticConnector::createConnection('pp_server', $form_state['values']['url'], $form_state['values']['server_title'], array(
    'username' => $form_state['values']['name'],
    'password' => $form_state['values']['pass'],
  ));

  // Update an existing configuration.
  if (isset($form_state['values']['tmid'])) {
    $config = new PPTaxonomyManagerConfig($form_state['values']['tmid']);
    $config->setTitle($form_state['values']['title']);
    $config->setConnectionId($connection->getId());
    $config->setProjectId($form_state['values']['project']);
    $tmid = $config->save();
    drupal_set_message(t('The connection for PoolParty Taxonomy Manager configuration %title has been updated.', array('%title' => $form_state['values']['title'])));
  }
  // Create a new configuration set.
  else {
    $config = new PPTaxonomyManagerConfig();
    $config->setTitle($form_state['values']['title']);
    $config->setConnectionId($connection->getId());
    $config->setProjectId($form_state['values']['project']);
    $tmid = $config->save();
    drupal_set_message(t('The PoolParty Taxonomy Manager configuration %title has been created.', array('%title' => $form_state['values']['title'])));
  }

  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid;
}

/**
 * A form to edit an existing PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_configuration_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  $form['tmid'] = array(
    '#type' => 'hidden',
    '#value' => $config->getId(),
  );

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the PowerTagging configuration.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $config->getTitle(),
    '#required' => TRUE,
  );

  // Get the project title of the currently configured project.
  $project_title = '<invalid project selected>';
  $pp_server_projects = $config->getConnection()->getApi('PPT')->getProjects();
  foreach ($pp_server_projects as $pp_server_project) {
    if ($pp_server_project->id == $config->getProjectId()) {
      $project_title = $pp_server_project->title;
      break;
    }
  }

  // Add information about the connection.
  $connection_markup = '';
  // Check the PoolParty server version if required.
  if (variable_get('semantic_connector_version_checking', TRUE)) {
    $api_version_info = $config->getConnection()->getVersionInfo('PPT');
    if (version_compare($api_version_info['installed_version'], $api_version_info['latest_version'], '<')) {
      $connection_markup .= '<div class="messages warning">' . t('The connected PoolParty server is not up to date. You are currently running version %installedversion, upgrade to version %latestversion to enjoy the new features.', array(
          '%installedversion' => $api_version_info['installed_version'],
          '%latestversion' => $api_version_info['latest_version'],
        )) . '</div>';
    }
  }
  $connection_markup .= '<p id="pp-taxonomy-manager-connection-info">' . t('Connected PoolParty server') . ': <b>' . $config->getConnection()
      ->getTitle() . ' (' . $config->getConnection()->getUrl() . ')</b><br />'
    . t('Selected project') . ': <b>' . $project_title . '</b><br />'
    . l(t('Change the connected PoolParty server or project'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/edit_connection') . '</p>';
  $form['pp_connection_markup'] = array(
    '#type' => 'markup',
    '#markup' => $connection_markup,
  );

  // Get the configuration of the selected PoolParty Taxonomy Manager.
  $configuration = $config->getConfig();
  $connected = array();

  // Get all available Drupal taxonomies.
  $taxonomies = taxonomy_get_vocabularies();

  // Get all taxonomies that are connected with a PowerTagging configuration.
  $powertaggings = array();
  $taxonomy_powertagging = array();
  if (module_exists('powertagging')) {
    $powertagging_configs = powertagging_config_load();
    foreach ($powertagging_configs as $powertagging_config) {
      $powertaggings[$powertagging_config->powertagging_id] = $powertagging_config->title;
      $vid = $powertagging_config->config['projects'][$powertagging_config->project_id]['taxonomy_id'];
      $taxonomy_powertagging[$vid][] = $powertagging_config->powertagging_id;
    }
  }

  // Create a table with all existing and possible connections between Drupal
  // taxonomies and PoolParty concepts schemes for the selected project.

  // Create the table rows for all available concept schemes not yet connected.
  $concept_schemes = $config->getConnection()
    ->getApi('PPT')
    ->getConceptSchemes($config->getProjectId());
  $rows = array();
  foreach ($concept_schemes as $scheme) {
    $operations = array();
    if (in_array($scheme->uri, $configuration['taxonomies'])) {
      $connected[$scheme->uri] = $scheme;
      continue;
    }
    $operations[] = '&lArr; ' . l(t('Import into Drupal'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $scheme->uri);
    $rows[] = array(
      '<div class="semantic-connector-italic">' . t('not yet connected') . '</div>',
      implode(' | ', $operations),
      l($scheme->title, $scheme->uri, array('attributes' => array('title' => (isset($scheme->descriptions) && !empty($scheme->descriptions)) ? $scheme->descriptions[0] : NULL))),
    );
  }

  // Create the table rows for all connected and disconnected Drupal taxonomies.
  foreach ($taxonomies as $taxonomy) {
    $operations = array();
    // Show only taxonomies that are not already connected with a
    // PoolParty project via PowerTagging.
    if (!isset($taxonomy_powertagging[$taxonomy->vid])) {
      if (isset($configuration['taxonomies'][$taxonomy->vid])) {
        // Create rows from connected taxonomies.
        if (isset($connected[$configuration['taxonomies'][$taxonomy->vid]])) {
          $scheme = $connected[$configuration['taxonomies'][$taxonomy->vid]];
          $operations[] = '&lArr; ' . l(t('Sync from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/sync/' . $taxonomy->vid);
          $operations[] = l(t('Disconnect from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/disconnect/' . $taxonomy->vid) . ' &rArr;';
          $concept_scheme = l($scheme->title, $scheme->uri, array(
            'absolut' => TRUE,
            'attributes' => array('title' => $scheme->descriptions[0]),
          ));
        }
        else {
          $operations[] = l(t('Disconnect from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/disconnect/' . $taxonomy->vid) . ' &rArr;';
          $concept_scheme = t('The concept scheme could not be found in PoolParty.<br />Make sure to delete the connection if the concept scheme was deleted on purpose.');
        }
        $rows[] = array(
          l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
          implode(' | ', $operations),
          $concept_scheme,
        );
      }
      else {
        // Create rows from disconnected taxonomies.
        $operations[] = l(t('Export to PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/export/' . $taxonomy->vid) . ' &rArr;';
        $rows[] = array(
          l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
          implode(' | ', $operations),
          '<div class="semantic-connector-italic">' . t('not yet connected') . '</div>',
        );
      }
    }
  }

  // Create the table for the connections.
  $table = array();
  $table['connections'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('Drupal taxonomy'),
      t('Operations'),
      t('PoolParty concept scheme'),
    ),
    '#rows' => $rows,
    '#attributes' => array(
      'id' => 'pp-taxonomy-manager-interconnection-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $table['#attached']['js'] = array(libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js');
  }

  $form['connections'] = array(
    '#type' => 'item',
    '#title' => '<h3 class="semantic-connector-table-title">' . t('Interconnection between the Drupal taxonomies and the PoolParty concept schemes') . '</h3>',
    '#markup' => drupal_render($table),
  );

  // Create a table with taxonomies already connected with a PoolParty project
  // via the PowerTagging module.

  $rows = array();
  foreach ($taxonomies as $taxonomy) {
    // Show only taxonomies that are already connected with a
    // PoolParty project via PowerTagging.
    if (isset($taxonomy_powertagging[$taxonomy->vid])) {
      $powertagging_links = array();
      $powertagging_update_links = array();
      foreach ($taxonomy_powertagging[$taxonomy->vid] as $powertagging_id) {
        $powertagging_links[] = l($powertaggings[$powertagging_id], 'admin/config/semantic-drupal/powertagging/' . $powertagging_id);
        $link_name = t('Update the taxonomy from "@powertagging"', array('@powertagging' => $powertaggings[$powertagging_id]));
        $powertagging_update_links[] = l($link_name, 'admin/config/semantic-drupal/powertagging/' . $powertagging_id . '/update-taxonomy', array(
          'query' => array('destination' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId()),
        ));
      }
      if (isset($configuration['taxonomies'][$taxonomy->vid])) {
        // Create rows from connected taxonomies created with this module.
        $rows[] = array(
          l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
          theme_item_list(array(
            'items' => $powertagging_links,
            'title' => '',
            'type' => 'ul',
            'attributes' => array(),
          )),
          l(t('Delete Synchronizer connection'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/disconnect/' . $taxonomy->vid),
        );
      }
      else {
        // Create rows from taxonomies that are not connected with this module.
        $rows[] = array(
          l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
          theme_item_list(array(
            'items' => $powertagging_links,
            'title' => '',
            'type' => 'ul',
            'attributes' => array(),
          )),
          theme_item_list(array(
            'items' => $powertagging_update_links,
            'title' => '',
            'type' => 'ul',
            'attributes' => array(),
          )),
        );
      }
    }
  }

  if (count($rows)) {
    $table = array();
    $table['powertagging'] = array(
      '#theme' => 'table',
      '#header' => array(
        t('Drupal taxonomy'),
        t('PowerTagging configurations'),
        t('Operation'),
      ),
      '#rows' => $rows,
      '#attributes' => array(
        'id' => 'pp-taxonomy-manager-powertagging-table',
        'class' => array('semantic-connector-tablesorter'),
      ),
    );

    $form['powertagging'] = array(
      '#type' => 'item',
      '#title' => '<h3 class="semantic-connector-table-title">' . t('Drupal taxonomies already connected with a PoolParty project via PowerTagging module') . '</h3>',
      '#markup' => drupal_render($table),
    );
  }

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['delete'] = array(
    '#type' => 'button',
    '#value' => t('Delete'),
    '#attributes' => array('onclick' => 'window.location.href = "' . url('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/delete') . '"; return false;'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/pp-taxonomy-manager'),
  );

  return $form;
}

/**
 * The submit handler for the PoolParty Taxonomy Manager configuration admin form.
 */
function pp_taxonomy_manager_configuration_form_submit($form, &$form_state) {
  $config = new PPTaxonomyManagerConfig($form_state['values']['tmid']);
  $old_title = $config->getTitle();
  $config->setTitle($form_state['values']['title']);
  $config->save();

  drupal_set_message(t('The connection for PoolParty Taxonomy Manager configuration %title has been updated.', array('%title' => $old_title)));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
}

/**
 * Add a PoolParty Taxonomy Manager configuration for a predefined PP connection + project.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param int $connection_id
 *   The ID of the PoolParty server connection
 * @param string $project_id
 *   The ID of the project on the PoolParty server
 *
 * @return array
 *   The Drupal form array of the confirmation form.
 */
function pp_taxonomy_manager_add_config_for_fixed_connection_form($form, &$form_state, $connection_id, $project_id) {
  $form_state['connection_id'] = $connection_id;
  $form_state['project_id'] = $project_id;

  $connection = SemanticConnector::getConnection('pp_server', $connection_id);
  $pp_config = $connection->getConfig();
  foreach ($pp_config['projects'] as $project) {
    if ($project->id == $project_id) {
      return confirm_form($form,
        t('Are you sure you want to create the PoolParty Taxonomy Manager configuration?'),
        (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/pp-taxonomy-manager'),
        'Selected PoolParty server: <b>' . $connection->getTitle() . '</b><br />Selected project: <b>' . $project->title . '</b>',
        t('Create configuration'));
    }
  }

  return NULL;
}

/**
 * Submit handler for pp_taxonomy_manager_add_config_for_fixed_connection_form().
 */
function pp_taxonomy_manager_add_config_for_fixed_connection_form_submit($form, &$form_state) {
  $connection = SemanticConnector::getConnection('pp_server', $form_state['connection_id']);
  $project_id = $form_state['project_id'];

  // Get the current list of projects from the server to make sure the project
  // still exists.
  $project_is_valid = FALSE;
  $projects = $connection->getApi('PPT')->getProjects();
  foreach ($projects as $project) {
    if ($project->id == $project_id) {
      $project_is_valid = TRUE;

      // Create a new configuration set.
      $config = new PPTaxonomyManagerConfig();
      $config_title = 'PoolParty Taxonomy Manager configuration for ' . $connection->getTitle() . ' (' . $project->title . ')';
      $config->setTitle($config_title);
      $config->setConnectionId($connection->getId());
      $config->setProjectId($project_id);
      $tmid = $config->save();

      drupal_set_message(t('PoolParty Taxonomy Manager configuration "%title" has been created.', array('%title' => $config_title)));
      // Drupal Goto to forward a destination if one is available.
      if (isset($_GET['destination'])) {
        $destination = $_GET['destination'];
        unset($_GET['destination']);
        drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid, array('query' => array('destination' => $destination)));
      }
      $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid;
      break;
    }
  }

  if (!$project_is_valid) {
    drupal_set_message(t('Project-ID "%projectid" was not found on PoolParty server "%ppservertitle"', array(
      '%projectid' => $project_id,
      '%ppservertitle' => $connection->getTitle(),
    )));
    $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
  }
}

/**
 * The confirmation-form for deleting a PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   The PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array of the confirmation form.
 */
function pp_taxonomy_manager_configuration_delete_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  $form_state['config'] = $config;
  return confirm_form($form,
    t('Are you sure you want to delete the PoolParty Taxonomy Manager configuration "%title"?', array('%title' => $config->getTitle())),
    'admin/config/semantic-drupal/pp-taxonomy-manager',
    t('This action cannot be undone.'),
    t('Delete configuration'));
}

/**
 * The submit handler for deleting a PoolParty Taxonomy Manager configuration.
 */
function pp_taxonomy_manager_configuration_delete_form_submit($form, &$form_state) {
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $config->delete();

  drupal_set_message(t('The connection for PoolParty Taxonomy Manager configuration %title has been deleted.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
}

/**
 * The confirmation-form for the export of a taxonomy to a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_export_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();
  $projects = $connection->getApi('PPT')->getProjects();
  $project = NULL;
  foreach ($projects as $project) {
    if ($project->id == $config->getProjectId()) {
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is already connected with a concept scheme.
  $configuration = $config->getConfig();
  if (isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is already connected, please select another one.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the sum of all terms from the taxonomy.
  $tree = taxonomy_get_tree($taxonomy->vid);
  $count = count($tree);

  $description = t('A new concept scheme will be created in the project %project and %count terms will be inserted.', array(
    '%project' => $project->title,
    '%count' => $count,
  ));
  $description .= '<br />' . t('This can take a while. Please wait until the export is finished.');
  $form['description'] = array(
    '#markup' => $description,
  );
  $form['concept_scheme_title'] = array(
    '#title' => t('Title of the new concept scheme'),
    '#type' => 'textfield',
    '#default_value' => $taxonomy->name,
    '#required' => TRUE,
  );

  // Language mapping.
  if (module_exists('i18n_taxonomy') && in_array($taxonomy->i18n_mode, array(
      I18N_MODE_LOCALIZE,
      I18N_MODE_TRANSLATE,
    ))) {
    $available_languages = language_list();
  }
  else {
    $available_languages = array(language_default());
  }
  $default_language = language_default('language');
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    $project_language_options[$project_language] = $project_language;
  }
  $form['languages'] = array(
    '#type' => 'item',
    '#title' => t('Map the Drupal languages with the PoolParty project languages'),
    '#description' => count($available_languages) > 1 ? t('The term-translations of the non-selected languages are not exported.') : '',
    '#tree' => TRUE,
  );
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $form['languages'][$lang->language] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang->name)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => ($lang->language == $default_language ? $project->defaultLanguage : ''),
        '#required' => ($lang->language == $default_language ? TRUE : FALSE),
        '#disabled' => ($lang->language == $default_language ? TRUE : FALSE),
      );
    }
  }

  $form['terms_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('Taxonomy terms per request'),
    '#description' => t('The number of terms, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished exporting ALL your terms, what results in a shorter duration of the bulk exporting process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk exporting process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  if (module_exists('i18n_taxonomy') && $taxonomy->i18n_mode == I18N_MODE_LOCALIZE) {
    $form['info'] = array(
      '#prefix' => '<p><label>' . t('Attention:') . '</label>',
      '#markup' => t('The translation mode of this taxonomy will be changed from "Localize" to "Translate" first before the export begins.'),
      '#suffix' => '</p>',
    );
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Export taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  return $form;
}

/**
 * Validate-Handler of the form to export terms to a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_export_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  // Check whether all languages are different.
  $languages = array_unique($values['languages']);
  if (count($values['languages']) != count($languages)) {
    form_set_error('languages', t('The selected languages must be different.'));
  }

  $concepts_per_request = $values['terms_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('terms_per_request', t('Only values in the range of 1 - 100 are allowed for field "Taxonomy terms per request"'));
  }
}

/**
 * The submit handler for the export confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_export_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];

  $terms_per_request = $form_state['values']['terms_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Set the correct translation mode for the taxonomy.
  $manager->setTranslationMode($taxonomy, $languages);

  // Add URI and alt. labels fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Create the new concept scheme in the PoolParty thesaurus.
  $scheme_uri = $manager->createConceptScheme($taxonomy, $form_state['values']['concept_scheme_title']);

  // Connect the taxonomy with the new concept scheme.
  $config->addConnection($taxonomy->vid, $scheme_uri, $languages);

  // Export all taxonomy terms.
  $manager->exportTaxonomyTerms($taxonomy, $scheme_uri, $languages, $terms_per_request);

  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the config of a taxonomy to a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_sync_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();
  $projects = $connection->getApi('PPT')->getProjects();
  $project = NULL;
  foreach ($projects as $project) {
    if ($project->id == $config->getProjectId()) {
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is connected with a concept scheme.
  $configuration = $config->getConfig();
  if (!isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is not connected, please export the taxonomy first.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $description = t('The taxonomy %taxonomy will be updated. It means that terms will be updated, deleted and/or created.', array('%taxonomy' => $taxonomy->name));
  $description .= '<br />' . t('This can take a while. Please wait until the synchronization is finished.');

  $form['description'] = array(
    '#markup' => $description,
  );

  // Language mapping.
  $available_languages = language_list();
  $default_language = language_default('language');
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    $project_language_options[$project_language] = $project_language;
  }
  $form['languages'] = array(
    '#type' => 'item',
    '#title' => t('Map the Drupal languages with the PoolParty project languages'),
    '#tree' => TRUE,
  );
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $form['languages'][$lang->language] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang->name)),
        '#description' => t('Select the PoolParty project language.'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => isset($configuration['languages'][$taxonomy->vid][$lang->language]) ? $configuration['languages'][$taxonomy->vid][$lang->language] : '',
        '#required' => ($lang->language == $default_language ? TRUE : FALSE),
        '#disabled' => ($lang->language == $default_language ? TRUE : FALSE),
      );
    }
  }

  $form['concepts_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('PoolParty concepts per request'),
    '#description' => t('The number of concepts, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished updating ALL your concepts, what results in a shorter duration of the bulk updating process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk updating process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Synchronize taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  return $form;
}

/**
 * Validate-Handler of the form to synchronize concepts with a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_sync_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  $default_language = language_default('language');
  if (!module_exists('i18n_taxonomy')) {
    foreach ($values['languages'] as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang) && $drupal_lang != LANGUAGE_NONE && $drupal_lang != $default_language) {
        $drupal_languages = language_list();
        form_set_error('languages][' . $drupal_lang, t('Language "%language" requires translation of taxonomies as it is not your default site language.<br /> Install and enable module "!internationalizationlink" and its sub-module "Taxonomy translation" to make multilingual tagging possible.', array(
          '%language' => $drupal_languages[$drupal_lang]->name,
          '!internationalizationlink' => l(t('Internationalization'), 'https://www.drupal.org/project/i18n', array('attributes' => array('target' => '_blank'))),
        )));
      }
    }
  }

  // Check whether all languages are different.
  $languages = array_unique($values['languages']);
  if (count($values['languages']) != count($languages)) {
    form_set_error('languages', t('The selected languages must be different.'));
  }

  $concepts_per_request = $values['concepts_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('concepts_per_request', t('Only values in the range of 1 - 100 are allowed for field "PoolParty concepts per request"'));
  }
}

/**
 * The submit handler for the synchronization confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_sync_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];

  $concepts_per_request = $values['concepts_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Set the correct translation mode for the taxonomy.
  $manager->setTranslationMode($taxonomy, $languages);

  // Add URI and alt. label fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Update the connection.
  $configuration = $config->getConfig();
  $scheme_uri = $configuration['taxonomies'][$taxonomy->vid];
  $config->updateConnection($taxonomy->vid, $scheme_uri, $languages);

  // Update all taxonomy terms.
  $manager->updateTaxonomyTerms($taxonomy, $scheme_uri, $languages, $concepts_per_request);
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the import of a taxonomy from a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_import_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  // Check if concept scheme URI is given and is a url.
  $scheme_uri = str_replace('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/', '', current_path());
  if (!valid_url($scheme_uri, TRUE)) {
    drupal_set_message(t('The URI from the selected concept scheme is not valid.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();
  $projects = $connection->getApi('PPT')->getProjects();
  $project = NULL;
  foreach ($projects as $project) {
    if ($project->id == $config->getProjectId()) {
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if concept scheme exists.
  $concept_schemes = $config->getConnection()
    ->getApi('PPT')
    ->getConceptSchemes($config->getProjectId());
  $concept_scheme = NULL;
  foreach ($concept_schemes as $scheme) {
    if ($scheme->uri == $scheme_uri) {
      $concept_scheme = $scheme;
      break;
    }
  }
  if (is_null($concept_scheme)) {
    drupal_set_message(t('The selected concept scheme does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is already connected with a concept scheme.
  $configuration = $config->getConfig();
  if (in_array($scheme_uri, $configuration['taxonomies'])) {
    drupal_set_message(t('The concept scheme %scheme is already connected, please select another one.', array('%scheme' => $concept_scheme->title)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the new taxonomy already exists in Drupal.
  $machine_name = PPTaxonomyManager::machineName($concept_scheme->title);
  $taxonomy_name = db_select('taxonomy_vocabulary', 't')
    ->fields('t', array('name'))
    ->condition('machine_name', $machine_name)
    ->execute()
    ->fetchField();

  $description = t('A new taxonomy will be created and all concepts from the concept scheme %scheme will be inserted as terms.', array('%scheme' => $concept_scheme->title));
  $description .= '<br />' . t('This can take a while. Please wait until the import is finished.');
  $form['description'] = array(
    '#markup' => $description,
  );
  $field_description = t('Please enter a name of a taxonomy, which does not yet exist.');
  if (!empty($taxonomy_name)) {
    $field_description .= '<br />' . t('The taxonomy %taxonomy (machine name: %machine_name) already exists. Its terms will be updated, deleted and/or created.', array(
        '%taxonomy' => $taxonomy_name,
        '%machine_name' => $machine_name,
      ));
  }
  $form['taxonomy_name'] = array(
    '#title' => t('Name of the new taxonomy'),
    '#type' => 'textfield',
    '#default_value' => $concept_scheme->title,
    '#description' => $field_description,
    '#required' => TRUE,
  );

  // Language mapping.
  $available_languages = language_list();
  $default_language = language_default('language');
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    $project_language_options[$project_language] = $project_language;
  }
  $form['languages'] = array(
    '#type' => 'item',
    '#title' => t('Map the Drupal languages with the PoolParty project languages'),
    '#tree' => TRUE,
  );
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $form['languages'][$lang->language] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang->name)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => ($lang->language == $default_language ? $project->defaultLanguage : ''),
        '#required' => ($lang->language == $default_language ? TRUE : FALSE),
        '#disabled' => ($lang->language == $default_language ? TRUE : FALSE),
      );
    }
  }

  $form['concepts_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('PoolParty concepts per request'),
    '#description' => t('The number of concepts, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished updating ALL your concepts, what results in a shorter duration of the bulk updating process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk updating process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => t('Import taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['concept_scheme'] = $concept_scheme;

  return $form;
}

/**
 * Validate-Handler of the form to import concepts from a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_import_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  $default_language = language_default('language');
  if (!module_exists('i18n_taxonomy')) {
    foreach ($values['languages'] as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang) && $drupal_lang != LANGUAGE_NONE && $drupal_lang != $default_language) {
        $drupal_languages = language_list();
        form_set_error('languages][' . $drupal_lang, t('Language "%language" requires translation of taxonomies as it is not your default site language.<br /> Install and enable module "!internationalizationlink" and its sub-module "Taxonomy translation" to make multilingual tagging possible.', array(
          '%language' => $drupal_languages[$drupal_lang]->name,
          '!internationalizationlink' => l(t('Internationalization'), 'https://www.drupal.org/project/i18n', array('attributes' => array('target' => '_blank'))),
        )));
      }
    }
  }

  // Check whether all languages are different.
  $languages = array_unique($values['languages']);
  if (count($values['languages']) != count($languages)) {
    form_set_error('languages', t('The selected languages must be different.'));
  }

  $concepts_per_request = $values['concepts_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('concepts_per_request', t('Only values in the range of 1 - 100 are allowed for field "PoolParty concepts per request"'));
  }
}

/**
 * The submit handler for the import confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_import_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $concept_scheme = $form_state['concept_scheme'];

  $concepts_per_request = $values['concepts_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Create the new taxonomy and set the translation mode.
  $taxonomy = $manager->createTaxonomy($concept_scheme, $values['taxonomy_name']);
  $manager->setTranslationMode($taxonomy, $languages);

  // Add URI and alt. labels fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Connect the new taxonomy with the concept scheme.
  $config->addConnection($taxonomy->vid, $concept_scheme->uri, $languages);

  // Import all concepts.
  $manager->updateTaxonomyTerms($taxonomy, $concept_scheme->uri, $languages, $concepts_per_request);
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the disconnection from a taxonomy to a PP server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_disconnect_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();
  $projects = $connection->getApi('PPT')->getProjects();
  $project = NULL;
  foreach ($projects as $project) {
    if ($project->id == $config->getProjectId()) {
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is connected with a concept scheme.
  $configuration = $config->getConfig();
  if (!isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is already disconnected, please select another one.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $description = t('The connection between the taxonomy %taxonomy and the PoolParty server will be deleted.', array('%taxonomy' => $taxonomy->name));
  $description .= '<br />' . t('Please wait until the disconnection is finished.');

  $form['description'] = array(
    '#prefix' => '<p>',
    '#markup' => $description,
    '#suffix' => '</p>',
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Disconnect taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  return $form;
}

/**
 * The submit handler for the disconnect confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_disconnect_form_submit($form, &$form_state) {
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];

  // Delete the connection.
  $config->deleteConnection($taxonomy->vid);

  // Delete all the logs and hash data.
  $manager = new PPTaxonomyManager($config);
  $manager->deleteSyncData($taxonomy->vid);

  $connection = $config->getConnection();
  drupal_set_message(t('The connection between the Drupal taxonomy %taxonomy and the PoolParty server %server has been deleted successfully.', array(
    '%taxonomy' => $taxonomy->name,
    '%server' => $connection->getTitle(),
  )));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * Returns all selected languages with the default language first.
 *
 * @param array $all_languages
 *   An array of languages:
 *    key = Drupal language
 *    value = PoolParty language.
 *
 * @return array
 *   All maped languages with the default language first.
 */
function pp_taxonomy_manager_order_languages($all_languages) {
  $default_language = language_default('language');
  $languages[$default_language] = $all_languages[$default_language];
  unset($all_languages[$default_language]);
  if (!empty($all_languages)) {
    foreach ($all_languages as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang)) {
        $languages[$drupal_lang] = $pp_lang;
      }
    }
  }

  return $languages;
}
